#include "Init.h"

#include "libcpp.h"
#include "InstallHooks.h"
#include "IFiltersManager.h"

//Variant of keyboard hook
#include "IOCompletionRoutine.h"
#include "APCRoutinePatch.h"

#include <exception>

PDEVICE_OBJECT g_pMainDeviceObject = NULL;

extern IFiltersManager *g_pFilterManager;

extern "C"
    NTSTATUS DriverEntry(PDRIVER_OBJECT pDriverObject, PUNICODE_STRING pRegistryPath)
{
    libcpp_init();

        for (unsigned int iCounter = 0; iCounter <= IRP_MJ_MAXIMUM_FUNCTION; ++iCounter){
            pDriverObject->MajorFunction[iCounter] = DefaultDispatchRoutine;
        }

	pDriverObject->DriverUnload = DriverUnload;

    ULONG ulDeviceSettings = FILE_DEVICE_SECURE_OPEN;
    UNICODE_STRING uDeviceName;
    DEVICE_TYPE ulDeviceType = 'DDFK';

    RtlInitUnicodeString(&uDeviceName, KBHookDeviceName);

    NTSTATUS lStatus = IoCreateDevice(pDriverObject, 
                                      0,
                                      &uDeviceName,
                                      ulDeviceType,
                                      ulDeviceSettings,
                                      FALSE,
                                      &g_pMainDeviceObject);

        if (!NT_SUCCESS(lStatus)){
            DbgPrint("[KBHookDriver]Cannot create common KBHookDriver device.\n");

            return(lStatus);
        }

    UNICODE_STRING uSymlink;

    RtlInitUnicodeString(&uSymlink, KBHookSymlink);

    IoDeleteSymbolicLink(&uSymlink);

    lStatus = IoCreateSymbolicLink(&uSymlink, &uDeviceName);

        if (!NT_SUCCESS(lStatus)){
            DbgPrint("[KBHookDriver]Cannot create device symbolic link.");

            return(lStatus);
        }

        try{
            InstallSimpleHooks(pDriverObject);
 			InstallKbdclassHook();
        }
        catch(std::exception& ex){
            DbgPrint(ex.what());

            return(STATUS_DRIVER_UNABLE_TO_LOAD);
        }

    return(lStatus);
}

void DriverUnload(PDRIVER_OBJECT pDriverObject){

	UninstallSimpleHooks();
	UninstallKbdclassHook();

	UNICODE_STRING uSymlink;

	RtlInitUnicodeString(&uSymlink, KBHookSymlink);

	IoDeleteSymbolicLink(&uSymlink);

	UNICODE_STRING uDeviceName;

	RtlInitUnicodeString(&uDeviceName, KBHookDeviceName);

	IoDeleteDevice(g_pMainDeviceObject);

	return;
}

NTSTATUS DefaultDispatchRoutine(PDEVICE_OBJECT pDeviceObject, PIRP pIRP){
	PIO_STACK_LOCATION pIOStack = IoGetCurrentIrpStackLocation(pIRP);
	NTSTATUS lStatus = STATUS_NOT_IMPLEMENTED;
	IIRPDispatcher *pCurrentDispatcher = g_pFilterManager->GetFilter(pDeviceObject);

		if (pCurrentDispatcher == NULL){
			return(STATUS_NOT_IMPLEMENTED);
		}

		try{
			switch(pIOStack->MajorFunction){
				case(IRP_MJ_READ):{

					//You can replace second parameter for APCRoutinePatch for APC patch hook method
					//lStatus = pCurrentDispatcher->DispatchRead(pIRP, IOCompletionRoutine);
					lStatus = pCurrentDispatcher->DispatchRead(pIRP, IOCompletionRoutine);

					break;
				}
				case(IRP_MJ_POWER):{

					lStatus = pCurrentDispatcher->DispatchPower(pIRP);

					break;
				}
				case(IRP_MJ_PNP):{

					lStatus = g_pFilterManager->DispatchPNP(pDeviceObject, pIRP);

					break;
				}
				default:{

					lStatus = pCurrentDispatcher->DispatchPassThrough(pIRP);

					break;
				}
			}
		}
		catch(std::exception& ex){
		}

    return(lStatus);
}