#include "CKBFilterManager.h"
#include "CKBFilterObject.h"

void CFilterManager::RegisterFilter(PDEVICE_OBJECT pAttachedFilter, 
									PDEVICE_OBJECT pNextDevice,
									IKBExternalObserver *pExternalObserver)
{

        if (this->IsExists(pAttachedFilter)){
            return;
        }

    IFilterObject *pFilterObject = new CKBFilterObject(pAttachedFilter, pNextDevice, pExternalObserver);

	m_filters.insert(FilterPair(pAttachedFilter, pFilterObject));

    return;
}

void CFilterManager::UnregisterFilter(PDEVICE_OBJECT pAttachedFilter){
	IFilterObject *pFilterObject = this->GetFilter(pAttachedFilter);

		if (pFilterObject != NULL){
			delete pFilterObject;
			m_filters.erase(pAttachedFilter);
		}

    return;
}

IFilterObject* CFilterManager::GetFilter(PDEVICE_OBJECT pDeviceObject){
	FiltersMap::iterator iIt = m_filters.find(pDeviceObject);

		if (iIt == m_filters.end()){
			return(NULL);
		}

    return(iIt->second);
}

bool CFilterManager::IsExists(PDEVICE_OBJECT pDeviceObject){
    return(this->GetFilter(pDeviceObject) != NULL);
}

NTSTATUS CFilterManager::DispatchPNP(PDEVICE_OBJECT pFilterObject, PIRP pIRP){
    PIO_STACK_LOCATION pIOStack = IoGetCurrentIrpStackLocation(pIRP);
    IFilterObject *pFilter = this->GetFilter(pFilterObject);
    PDEVICE_OBJECT pNextDevice = pFilter->GetNextDevice();

        switch(pIOStack->MinorFunction){
            case(IRP_MN_SURPRISE_REMOVAL):{
                
                //pFilter->PNPRemoveDevice();

                break;
            }
            case(IRP_MN_REMOVE_DEVICE):{

                this->UnregisterFilter(pFilterObject);

                break;
            }
        }

    IoSkipCurrentIrpStackLocation(pIRP);

    return(IoCallDriver(pNextDevice, pIRP));
}

CFilterManager::~CFilterManager(void){
	
		for (FiltersMap::iterator iIt = m_filters.begin(); iIt != m_filters.end(); ++iIt){
			delete iIt->second;
		}

}