#include "CKBFilterObject.h"

#include "stdexcept"

CKBFilterObject::CKBFilterObject(PDEVICE_OBJECT pKBFilterDevice, 
								 PDEVICE_OBJECT pNextDevice,
								 IKBExternalObserver *pExternalObserver):
	m_pKBFilterDevice(pKBFilterDevice),
	m_pNextDevice(pNextDevice),
	m_pendingCompleteEvent(NotificationEvent, TRUE),
	m_pExternalObserver(pExternalObserver)
{
}

PDEVICE_OBJECT CKBFilterObject::GetSelf(void){
	return(m_pKBFilterDevice);
}

PDEVICE_OBJECT CKBFilterObject::GetNextDevice(void){
	return(m_pNextDevice);
}

const ULONG CKBFilterObject::GetPendingPacketsCount(void) const{
	return(m_ulPendingPackets);
}

IKBExternalObserver* CKBFilterObject::GetDeviceObserver(void){
	return(m_pExternalObserver);
}

void CKBFilterObject::AttachFilter(void){

	m_pNextDevice = IoAttachDeviceToDeviceStack(m_pKBFilterDevice, m_pNextDevice);

		if (m_pNextDevice == NULL){
			throw(std::runtime_error("[KBHookDriver]Cannot attach filter."));
		}

	m_bIsAttached = true;

	return;
}

const bool CKBFilterObject::IsAttached(void){
	return(m_bIsAttached);
}

void CKBFilterObject::AddPendingPacket(PIRP pIRP){
	CAutoSpinLockGuard guard(&m_spinLock);

	m_pendingIRPs.insert(pIRP);

		if (m_pendingCompleteEvent.IsSet()){
			m_pendingCompleteEvent.Reset();
		}

	return;
}

void CKBFilterObject::RemovePendingPacket(PIRP pIRP){
	CAutoSpinLockGuard guard(&m_spinLock);

	m_pendingIRPs.erase(pIRP);

		if ((m_pendingIRPs.size() == 0) && !m_pendingCompleteEvent.IsSet()){
			m_pendingCompleteEvent.Set();
		}

	return;
}

NTSTATUS CKBFilterObject::DispatchPassThrough(PIRP pIRP){

	IoSkipCurrentIrpStackLocation(pIRP);

	return(IoCallDriver(m_pNextDevice, pIRP));
}

NTSTATUS CKBFilterObject::DispatchRead(PIRP pIRP, PDISPATCHCALLBACK pCallback){

	IoCopyCurrentIrpStackLocationToNext(pIRP);

	pCallback(this, pIRP);

	return(IofCallDriver(m_pNextDevice, pIRP));
}

NTSTATUS CKBFilterObject::DispatchPower(PIRP pIRP){

	PoStartNextPowerIrp(pIRP);
	IoSkipCurrentIrpStackLocation(pIRP);

	return(PoCallDriver(m_pNextDevice, pIRP));
}

void CKBFilterObject::DetachFilter(void){

		if (this->IsAttached()){
			m_pendingCompleteEvent.Wait();
			IoDetachDevice(m_pNextDevice);
		}

	return;
}

CKBFilterObject::~CKBFilterObject(void){

	this->DetachFilter();

	IoDeleteDevice(m_pKBFilterDevice);
}
