#ifndef __C_KB_FILTER_OBJECT_H__
#define __C_KB_FILTER_OBJECT_H__

#include "IFilterObject.h"
#include "Guards.h"

#include <set>

class CKBFilterObject:
	public IFilterObject
{
	public:
		CKBFilterObject(PDEVICE_OBJECT pKBFilterDevice, 
						PDEVICE_OBJECT pNextDevice,
						IKBExternalObserver *pExternalObserver);

		PDEVICE_OBJECT GetSelf(void);
		PDEVICE_OBJECT GetNextDevice(void);
		const ULONG GetPendingPacketsCount(void) const;
		IKBExternalObserver* GetDeviceObserver(void);

		void AttachFilter(void);
		const bool IsAttached(void);

		void AddPendingPacket(PIRP pIRP);
		void RemovePendingPacket(PIRP pIRP);

		//IRP dispatchers
		NTSTATUS DispatchPassThrough(PIRP pIRP);
		NTSTATUS DispatchRead(PIRP pIRP, PDISPATCHCALLBACK pCallback);
		NTSTATUS DispatchPower(PIRP pIRP);

		~CKBFilterObject(void);
	private:
		PDEVICE_OBJECT m_pKBFilterDevice;
		PDEVICE_OBJECT m_pNextDevice;
		volatile LONG m_ulPendingPackets;
		bool m_bIsAttached;
		std::set<PIRP> m_pendingIRPs;
		CSpinLockWrapper m_spinLock;
		CNativeEventWrapper m_pendingCompleteEvent;
		IKBExternalObserver *m_pExternalObserver;

		void DetachFilter(void);
};

#endif