#include "CKbdclassHook.h"
#include "KbdclassDefs.h"

CKbdclassHook::CKbdclassHook(const std::wstring& wsDriverName, 
							 PDRIVER_DISPATCH pHookCallback,
							 IKBExternalObserverEx *pObserver):
	m_wsClassDrvName(wsDriverName),
	m_pHookCallback(pHookCallback),
	m_pObserver(pObserver)
{
}

NTSTATUS CKbdclassHook::Call_DispatchRead(PDEVICE_OBJECT pDeviceObject, PIRP pIRP){
	PKBDCLASS_DEVICE_EXTENSION pDevExt = (PKBDCLASS_DEVICE_EXTENSION)pDeviceObject->DeviceExtension;
	
		if (pIRP->IoStatus.Status == STATUS_SUCCESS){
			PKEYBOARD_INPUT_DATA key = (PKEYBOARD_INPUT_DATA)pIRP->UserBuffer;
			KEY_STATE_DATA keyData;

			keyData.pusScanCode = &key->MakeCode;

				if (key->Flags & KEY_BREAK){
					keyData.bPressed = false;
				}
				else{
					keyData.bPressed = true;
				}

			m_pObserver->OnProcessEvent(pDevExt->TopPort, keyData);
		}

	return(m_pOriginalDispatchRead(pDeviceObject, pIRP));
}

bool CKbdclassHook::IsHooked(void) const{
	return(m_bEnabled);
}

void CKbdclassHook::Hook(void){
	UNICODE_STRING usKbdClassDriverName;

	RtlInitUnicodeString(&usKbdClassDriverName, m_wsClassDrvName.c_str());

	NTSTATUS lStatus = ObReferenceObjectByName(&usKbdClassDriverName,
											   OBJ_CASE_INSENSITIVE,
											   NULL,
											   0,
											   (POBJECT_TYPE)IoDriverObjectType,
											   KernelMode,
											   NULL,
											   (PVOID*)&m_pClassDriver);

		if (!NT_SUCCESS(lStatus)){
			throw(std::exception("[KBHookLib]Cannot get driver object by name."));
		}

	KIRQL oldIRQL;

	KeRaiseIrql(HIGH_LEVEL, &oldIRQL);

	m_pOriginalDispatchRead = m_pClassDriver->MajorFunction[IRP_MJ_READ];
	m_pClassDriver->MajorFunction[IRP_MJ_READ] = m_pHookCallback;

	m_bEnabled = true;

	KeLowerIrql(oldIRQL);

	return;
}

void CKbdclassHook::Unhook(void){
	KIRQL oldIRQL;

	KeRaiseIrql(HIGH_LEVEL, &oldIRQL);

	m_pClassDriver->MajorFunction[IRP_MJ_READ] = m_pOriginalDispatchRead;

	m_bEnabled = false;

	KeLowerIrql(oldIRQL);

	return;
}

CKbdclassHook::~CKbdclassHook(void){

		if (this->IsHooked()){
			this->Unhook();
		}

}