#include "IOCompletionRoutine.h"

extern "C"{
	#include <ntddkbd.h>
}

#include <exception>

static NTSTATUS OnReadCompletion(PDEVICE_OBJECT pDeviceObject, PIRP pIRP, PVOID pContext);

void IOCompletionRoutine(IIRPProcessor *pContext, PIRP pIRP){

	IoCopyCurrentIrpStackLocationToNext(pIRP);
	IoSetCompletionRoutine(pIRP, OnReadCompletion, pContext, TRUE, TRUE, TRUE);

	pContext->AddPendingPacket(pIRP);

	return;
}

static NTSTATUS OnReadCompletion(PDEVICE_OBJECT pDeviceObject, PIRP pIRP, PVOID pContext){
	IIRPProcessor *pIRPProcessor = (IIRPProcessor*)pContext;

	if (pIRP->IoStatus.Status == STATUS_SUCCESS){
		PKEYBOARD_INPUT_DATA keys = (PKEYBOARD_INPUT_DATA)pIRP->AssociatedIrp.SystemBuffer;
		unsigned int iKeysCount = pIRP->IoStatus.Information / sizeof(KEYBOARD_INPUT_DATA);

		for (unsigned int iCounter = 0; iCounter < iKeysCount; ++iCounter){
			KEY_STATE_DATA keyData;

			keyData.pusScanCode = &keys[iCounter].MakeCode;

			if (keys[iCounter].Flags & KEY_BREAK){
				keyData.bPressed = false;
			}
			else{
				keyData.bPressed = true;
			}

			try{
				pIRPProcessor->GetDeviceObserver()->OnProcessEvent(keyData);
				keys[iCounter].Flags = keyData.bPressed ? KEY_MAKE : KEY_BREAK;
			}
			catch(std::exception& ex){
				DbgPrint("[KBHookLib]%s\n", ex.what());
			}
		}
	}

	if(pIRP->PendingReturned){
		IoMarkIrpPending(pIRP);
	}

	pIRPProcessor->RemovePendingPacket(pIRP);

	return(pIRP->IoStatus.Status);
}