#ifndef __UTILS_WRAPPERS_H_
#define __UTILS_WRAPPERS_H_

extern "C"{
    #include <ntifs.h>
}

#include <list>

//-------------------------------------------------------------------------------------------------
class CSpinLockWrapper{
    public:
        CSpinLockWrapper(void);
        explicit CSpinLockWrapper(KSPIN_LOCK& spinLock);

        void Enter(void);
        void Leave(void);
    private:
        KSPIN_LOCK m_spinLock;
        KIRQL m_irql;

        CSpinLockWrapper(CSpinLockWrapper&);
        CSpinLockWrapper& operator =(CSpinLockWrapper&);
};
//-------------------------------------------------------------------------------------------------
class CNativeEventWrapper{
    public:
        CNativeEventWrapper(const EVENT_TYPE eventType, 
                            const BOOLEAN bInitState = FALSE);
        explicit CNativeEventWrapper(KEVENT& eventObject);

        void Set(void);
        void Reset(void);
        const bool IsSet(void);
        void Wait(PLARGE_INTEGER liTimeout = NULL);
        KEVENT& GetObject(void);

    private:
        KEVENT m_eventObject;
};
//-------------------------------------------------------------------------------------------------
class CAutoSpinLockGuard;

template<class T>
	class CLockableListWrapper
{
	public:

		void AddListHead(const T& data){
			CAutoSpinLockGuard guard(&m_lock);
			
			m_list.push_front(data);

			return;
		}

		void AddListTail(const T& data){
			CAutoSpinLockGuard guard(&m_lock);

			m_list.push_back(data);

			return;
		}

		T PopListHead(void){
			CAutoSpinLockGuard guard(&m_lock);
			T tmpData = m_list.front();

			m_list.pop_front();

			return(tmpData);
		}

		T PopListTail(void){
			CAutoSpinLockGuard guard(&m_lock);
			T tmpData = m_list.back();

			m_list.pop_back();

			return(tmpData);
		}

		void RemoveListHead(void){
			CAutoSpinLockGuard guard(&m_lock);

			m_list.pop_front();

			return;
		}

		void RemoveListTail(void){
			CAutoSpinLockGuard guard(&m_lock);

			m_list.pop_back();

			return;
		}

		T& GetListHead(void){
			return(m_list.front());
		}

		T& GetListTail(void){
			return(m_list.back());
		}

		const bool IsEmpty(void) const{
			return(m_list.empty());
		}

	private:
		std::list<T> m_list;
		CSpinLockWrapper m_lock;
};
//-------------------------------------------------------------------------------------------------

#endif