/*
    This is a C++ run-time library for Windows kernel-mode drivers.
    Copyright (C) 2004 Bo Brantn.
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

//eXploit: mf for <typeinfo.h> from previous libcpp version. Exceptions added.

#if !defined(TYPEINFO_H)
#define TYPEINFO_H

class type_info {
public:
    virtual ~type_info();
    bool operator==(const type_info&) const;
    bool operator!=(const type_info&) const;
    bool before(const type_info&) const;
    const char* name() const;
private:
    void* m_data;
    char m_name[1];
    type_info(const type_info&);
    type_info& operator=(const type_info&);
};

extern "C"{
#include <ntifs.h>
}

#include "exception"

_STD_BEGIN

class bad_cast : public exception {
public:
     bad_cast(const char * _Message = "bad cast")
        : exception(_Message)
    {}
     bad_cast(const bad_cast &_That)
        : exception(_That)
    {}
    virtual  ~bad_cast()
    {}
};

class bad_typeid : public exception {
public:
     bad_typeid(const char * _Message = "bad typeid")
        : exception(_Message)
    {}
     bad_typeid(const bad_typeid &_That)
        : exception(_That)
    {}
    virtual  ~bad_typeid()
    {}
};

class __non_rtti_object : public bad_typeid {
public:
     __non_rtti_object(const char * _Message = "non RTTI object")
        : bad_typeid(_Message)
    {}
     __non_rtti_object(const __non_rtti_object &_That)
        : bad_typeid(_That)
    {}
    virtual  ~__non_rtti_object()
    {}

};

_STD_END

#endif // !defined(TYPEINFO_H)
